@@ -1,6 +1,6 @@
 /* -*-pgsql-c-*- */
 /*
- * $Header: /home/t-ishii/repository/pgpool/main.c,v 1.3 2003/06/26 08:49:35 t-ishii Exp $
+ * $Header: /home/t-ishii/repository/pgpool/main.c,v 1.4 2003/07/05 06:32:33 t-ishii Exp $
  *
  * pgpool: a language independent connection pool server for PostgreSQL 
  * written by Tatsuo Ishii
@@ -208,7 +208,7 @@ static void usage(void)
 	fprintf(stderr, "  -n: don't run in daemon mode. does not detatch control tty\n");
 	fprintf(stderr, "  -d: debug mode. lots of debug information will be printed\n");
 	fprintf(stderr, "  stop: stop pgpool\n");
-	fprintf(stderr, "  -f: print this help\n");
+	fprintf(stderr, "  -h: print this help\n");
 }
 
 /*
