@@ -1,6 +1,6 @@
 /* -*-pgsql-c-*- */
 /*
- * $Header: /home/t-ishii/repository/pgpool/pool_connection_pool.c,v 1.5 2003/07/05 06:32:33 t-ishii Exp $
+ * $Header: /home/t-ishii/repository/pgpool/pool_connection_pool.c,v 1.6 2003/07/20 14:44:37 t-ishii Exp $
  *
  * pgpool: a language independent connection pool server for PostgreSQL 
  * written by Tatsuo Ishii
@@ -33,6 +33,7 @@
 
 #include <stdio.h>
 #include <errno.h>
+#include <signal.h>
 #include <string.h>
 #include <unistd.h>
 
@@ -60,7 +61,7 @@ int pool_init_cp(void)
 }
 
 /*
-* find connection bhy user and database
+* find connection by user and database
 */
 POOL_CONNECTION_POOL *pool_get_cp(char *user, char *database)
 {
@@ -80,6 +81,8 @@ POOL_CONNECTION_POOL *pool_get_cp(char *user, char *database)
 			strcmp(p->user, user) == 0 &&
 			strcmp(p->database, database) == 0)
 		{
+			/* mark this connection is under use */
+			p->closetime = 0;
 			return p;
 		}
 		p++;
@@ -87,8 +90,13 @@ POOL_CONNECTION_POOL *pool_get_cp(char *user, char *database)
 	return NULL;
 }
 
+/*
+ * disconnect and release a connection to the database
+ */
 void pool_discard_cp(char *user, char *database)
 {
+	int i;
+
 	POOL_CONNECTION_POOL *p = pool_get_cp(user, database);
 
 	if (p == NULL)
@@ -151,6 +159,7 @@ POOL_CONNECTION_POOL *pool_create_cp(char *user, char *database)
 				return NULL;
 			}
 			p->con = pool_open(fd);
+			p->closetime = 0;
 			return p;
 		}
 		p++;
@@ -158,6 +167,93 @@ POOL_CONNECTION_POOL *pool_create_cp(char *user, char *database)
 	return NULL;
 }
 
+/*
+ * set backend connection close timer
+ */
+void pool_connection_pool_timer(POOL_CONNECTION_POOL *backend)
+{
+	POOL_CONNECTION_POOL *p = pool_connection_pool;
+	int i;
+
+	pool_debug("pool_connection_pool_timer: called");
+
+	backend->closetime = time(NULL);		/* set connection close time */
+
+	if (pool_config.connection_life_time == 0)
+		return;
+
+	/* look for any other timeout */
+	for (i=0;i<pool_config.max_pool;i++, p++)
+	{
+		if (p->user == NULL)
+			continue;
+
+		if (p != backend && p->closetime)
+			return;
+	}
+
+	/* no other timer found. set my timer */
+	pool_debug("pool_connection_pool_timer: set alarm after %d seconds", pool_config.connection_life_time);
+	signal(SIGALRM, pool_backend_timer_handler);
+	alarm(pool_config.connection_life_time);
+}
+
+/*
+ * backend connection close timer handler
+ */
+RETSIGTYPE pool_backend_timer_handler(int sig)
+{
+#define TMINTMAX 0x7fffffff
+
+	POOL_CONNECTION_POOL *p = pool_connection_pool;
+	int i;
+	time_t now;
+	time_t nearest = TMINTMAX;
+
+	now = time(NULL);
+
+	pool_debug("pool_backend_timer_handler called at %d", now);
+
+	for (i=0;i<pool_config.max_pool;i++, p++)
+	{
+		if (p->user == NULL)
+			continue;
+
+		/* timer expire? */
+		if (p->closetime)
+		{
+			pool_debug("pool_backend_timer_handler: expire time: %d",
+					   p->closetime+pool_config.connection_life_time);
+
+			if (now >= (p->closetime+pool_config.connection_life_time))
+			{
+				/* discard expired connection */
+				pool_debug("pool_backend_timer_handler: expires user %s database %s", p->user, p->database);
+				free(p->user);
+				free(p->database);
+				pool_close(p->con);
+				memset(p, 0, sizeof(POOL_CONNECTION_POOL));
+			}
+			else
+			{
+				/* look for nearest timer */
+				if (p->closetime < nearest)
+					nearest = p->closetime;
+			}
+		}
+	}
+
+	/* any remaining timer */
+	if (nearest != TMINTMAX)
+	{
+		nearest = pool_config.connection_life_time - (now - nearest);
+		if (nearest <= 0)
+		  nearest = 1;
+		signal(SIGALRM, pool_backend_timer_handler);
+		alarm(nearest);
+	}
+}
+
 static  int connect_inet_domain_socket(POOL_CONNECTION_POOL *p)
 {
 	int fd;
